@@ -1,21 +1,19 @@
 use crate::args::Args;
 use crate::exec::{self, Status};
 use crate::fmt::color_repo;
-use crate::repo;
-
 use crate::util::get_provider;
+use crate::{alpm_debug_enabled, help, printtr, repo};
 
 use std::env::consts::ARCH;
-use std::env::var;
+use std::env::{remove_var, set_var, var};
 use std::fmt;
 use std::fs::File;
 use std::io::{stdin, BufRead};
 use std::path::{Path, PathBuf};
-use std::time::Duration;
 
-use alpm::{set_questioncb, Question, SigLevel, Usage};
-#[cfg(feature = "git")]
-use alpm::{DownloadEvent, DownloadResult};
+use alpm::{
+    AnyDownloadEvent, AnyQuestion, Depend, DownloadEvent, DownloadResult, LogLevel, Question,
+};
 use ansi_term::Color::{Blue, Cyan, Green, Purple, Red, Yellow};
 use ansi_term::Style;
 use anyhow::{anyhow, bail, ensure, Context, Error, Result};
@@ -23,13 +21,10 @@ use atty::Stream::Stdout;
 use cini::{Callback, CallbackKind, Ini};
 use globset::{Glob, GlobSet, GlobSetBuilder};
 use nix::unistd::dup2;
-use once_cell::sync::OnceCell;
 use std::os::unix::io::AsRawFd;
+use tr::tr;
 use url::Url;
 
-static COLORS: OnceCell<Colors> = OnceCell::new();
-pub static NO_CONFIRM: OnceCell<bool> = OnceCell::new();
-
 #[derive(Debug, Default)]
 pub struct Alpm {
     alpm: Option<alpm::Alpm>,
@@ -98,6 +93,8 @@ pub struct Colors {
     pub new_version: Style,
     pub number_menu: Style,
     pub group: Style,
+    pub stats_line_separator: Style,
+    pub stats_value: Style,
 }
 
 impl From<&str> for Colors {
@@ -118,7 +115,7 @@ impl Colors {
             error: Style::new().fg(Red),
             warning: Style::new().fg(Yellow),
             bold: Style::new().bold(),
-            upgrade: Style::new().fg(Green),
+            upgrade: Style::new().fg(Green).bold(),
             base: Style::new().fg(Blue),
             action: Style::new().fg(Blue).bold(),
             sl_repo: Style::new().fg(Purple).bold(),
@@ -138,6 +135,8 @@ impl Colors {
             new_version: Style::new().fg(Green),
             number_menu: Style::new().fg(Purple),
             group: Style::new().fg(Blue).bold(),
+            stats_line_separator: Style::new().fg(Blue).bold(),
+            stats_value: Style::new().fg(Cyan),
         }
     }
 }
@@ -171,18 +170,26 @@ pub trait ConfigEnum: Sized + PartialEq + Copy + Clone + fmt::Debug + 'static {
                 .map(|v| v.0)
                 .collect::<Vec<&str>>()
                 .join("|");
-            bail!(
-                "invalid value '{}' for key '{}', expected: {}",
-                value,
-                key,
-                okvalues
-            )
+            bail!(tr!(
+                "invalid value '{val}' for key '{key}', expected: {exp}",
+                val = value,
+                key = key,
+                exp = okvalues
+            ))
         }
     }
 }
 
 type ConfigEnumValues<T> = &'static [(&'static str, T)];
 
+#[derive(Debug, SmartDefault, PartialEq, Eq)]
+pub enum Sign {
+    #[default]
+    No,
+    Yes,
+    Key(String),
+}
+
 #[derive(Debug, Clone, Copy, PartialEq, Eq)]
 pub enum Op {
     ChrootCtl,
@@ -316,7 +323,11 @@ pub struct Config {
 
     #[default(Op::Yay)]
     pub op: Op,
+
+    #[cfg(not(feature = "mock"))]
     pub raur: raur::Handle,
+    #[cfg(feature = "mock")]
+    pub raur: crate::mock::Mock,
     #[default(aur_fetch::Handle::with_cache_dir(""))]
     pub fetch: aur_fetch::Handle,
     pub cache: raur::Cache,
@@ -337,6 +348,7 @@ pub struct Config {
     pub config_path: Option<PathBuf>,
 
     pub news: u32,
+    pub stats: bool,
     pub gendb: bool,
 
     #[default(YesNoAll::No)]
@@ -376,6 +388,8 @@ pub struct Config {
     pub print: bool,
     pub news_on_upgrade: bool,
     pub comments: bool,
+    pub sign: Sign,
+    pub sign_db: Sign,
 
     #[default = "makepkg"]
     pub makepkg_bin: String,
@@ -415,16 +429,15 @@ pub struct Config {
     pub pacman_conf: Option<String>,
 
     pub repos: LocalRepos,
-    pub local: bool,
     #[default(Path::new("/var/lib/aurbuild/").join(ARCH))]
     pub chroot_dir: PathBuf,
     pub chroot: bool,
-    pub move_pkgs: bool,
     pub install: bool,
+    pub uninstall: bool,
     pub update: bool,
     pub quiet: bool,
     pub list: bool,
-    pub delete: bool,
+    pub delete: u32,
 
     //pacman
     pub db_path: Option<String>,
@@ -435,6 +448,7 @@ pub struct Config {
 
     pub ignore: Vec<String>,
     pub ignore_group: Vec<String>,
+    pub assume_installed: Vec<String>,
 }
 
 impl Ini for Config {
@@ -444,6 +458,9 @@ impl Ini for Config {
         let err = match cb.kind {
             CallbackKind::Section(section) => {
                 self.section = Some(section.to_string());
+                if !matches!(section, "options" | "bin" | "env") {
+                    eprintln!("{}", tr!("error: unknown section '{}'", section));
+                }
                 Ok(())
             }
             CallbackKind::Directive(_, key, value) => self.parse_directive(key, value),
@@ -456,10 +473,11 @@ impl Ini for Config {
 
 impl Config {
     pub fn new() -> Result<Self> {
-        let cache = dirs::cache_dir().ok_or_else(|| anyhow!("failed to find cache directory"))?;
+        let cache =
+            dirs::cache_dir().ok_or_else(|| anyhow!(tr!("failed to find cache directory")))?;
         let cache = cache.join("paru");
         let config =
-            dirs::config_dir().ok_or_else(|| anyhow!("failed to find config directory"))?;
+            dirs::config_dir().ok_or_else(|| anyhow!(tr!("failed to find config directory")))?;
         let config = config.join("paru");
 
         let build_dir = cache.join("clone");
@@ -471,11 +489,11 @@ impl Config {
         let cols = term_size::dimensions_stdout().map(|v| v.0);
 
         let mut config = Self {
-            devel_path,
             cols,
-            cache_dir,
             color,
             build_dir,
+            cache_dir,
+            devel_path,
             ..Self::default()
         };
 
@@ -483,8 +501,7 @@ impl Config {
             let path = PathBuf::from(conf);
             ensure!(
                 path.exists(),
-                "config file '{}' does not exist",
-                path.display()
+                tr!("config file '{}' does not exist", path.display())
             );
             config.config_path = Some(path);
         } else if config_path.exists() {
@@ -514,7 +531,8 @@ impl Config {
         self.globals.as_str()
     }
 
-    pub fn parse_args<S: AsRef<str>, I: Iterator<Item = S>>(&mut self, iter: I) -> Result<()> {
+    pub fn parse_args<S: AsRef<str>, I: IntoIterator<Item = S>>(&mut self, iter: I) -> Result<()> {
+        let iter = iter.into_iter();
         let mut iter = iter.peekable();
         let mut op_count = 0;
         let mut end_of_ops = false;
@@ -530,7 +548,10 @@ impl Config {
                 iter.next();
             }
 
-            ensure!(op_count <= 1, "only one operation may be used at a time");
+            ensure!(
+                op_count <= 1,
+                tr!("only one operation may be used at a time")
+            );
         }
 
         if let Some((i, _)) = self.targets.iter().enumerate().find(|t| t.1 == "-") {
@@ -549,7 +570,7 @@ impl Config {
         if self.help {
             match self.op {
                 Op::GetPkgBuild | Op::Show | Op::Yay => {
-                    help();
+                    help::help();
                     std::process::exit(0);
                 }
                 _ => {
@@ -571,22 +592,23 @@ impl Config {
             self.color = Colors::from("auto");
         }
 
-        ensure!(
-            COLORS.set(self.color).is_ok(),
-            "failed to initialize colors"
-        );
-        ensure!(
-            NO_CONFIRM.set(self.no_confirm).is_ok(),
-            "failed to initialize noconfirm"
-        );
+        #[cfg(not(feature = "mock"))]
+        {
+            use std::time::Duration;
 
-        let ver = option_env!("PARU_VERSION").unwrap_or(env!("CARGO_PKG_VERSION"));
+            let ver = option_env!("PARU_VERSION").unwrap_or(env!("CARGO_PKG_VERSION"));
+            let client = reqwest::Client::builder()
+                .tcp_keepalive(Duration::new(15, 0))
+                .user_agent(format!("paru/{}", ver))
+                .build()?;
 
-        let client = reqwest::Client::builder()
-            .tcp_keepalive(Duration::new(15, 0))
-            .user_agent(format!("paru/{}", ver))
-            .build()?;
-        self.raur = raur::Handle::new_with_settings(client, self.aur_url.join("rpc")?.as_str());
+            self.raur = raur::Handle::new_with_settings(client, self.aur_url.join("rpc")?.as_str());
+        }
+
+        #[cfg(feature = "mock")]
+        {
+            self.raur = crate::mock::Mock::new()?;
+        }
 
         self.fetch = aur_fetch::Handle {
             git: self.git_bin.clone().into(),
@@ -615,11 +637,23 @@ impl Config {
 
             for repo in repos {
                 if !self.pacman.repos.iter().any(|r| r.name == repo.name()) {
-                    bail!("can not find local repo '{}' in pacman.conf", repo.name());
+                    bail!(tr!(
+                        "can not find local repo '{}' in pacman.conf",
+                        repo.name()
+                    ));
                 }
             }
         }
         self.no_warn = self.no_warn_builder.build()?;
+
+        if !self.assume_installed.is_empty() && !self.chroot {
+            self.mflags.push("-d".to_string());
+        }
+
+        if self.chroot {
+            remove_var("PKGEXT");
+        }
+
         Ok(())
     }
 
@@ -638,61 +672,31 @@ impl Config {
     }
 
     pub fn init_alpm(&mut self) -> Result<()> {
-        let mut alpm = alpm::Alpm::new(&*self.pacman.root_dir, &*self.pacman.db_path)
-            .with_context(|| {
-                format!(
-                    "failed to initialize alpm: root={} dbpath={}",
-                    self.pacman.root_dir, self.pacman.db_path
-                )
-            })?;
+        let mut alpm = alpm_utils::alpm_with_conf(&self.pacman).with_context(|| {
+            tr!(
+                "failed to initialize alpm: root={} dbpath={}",
+                self.pacman.root_dir,
+                self.pacman.db_path
+            )
+        })?;
 
-        set_questioncb!(alpm, question);
+        alpm.set_question_cb((self.no_confirm, self.color), question);
+        alpm.set_dl_cb((), download);
+        alpm.set_log_cb(self.color, log);
 
-        #[cfg(feature = "git")]
-        alpm::set_dlcb!(alpm, download);
-
-        for repo in &self.pacman.repos {
-            let db = alpm.register_syncdb_mut(&*repo.name, SigLevel::NONE)?;
-            db.set_servers(repo.servers.iter())?;
-
-            let mut usage = Usage::NONE;
-
-            for v in &repo.usage {
-                match v.as_str() {
-                    "Sync" => usage |= Usage::SYNC,
-                    "Search" => usage |= Usage::SEARCH,
-                    "Install" => usage |= Usage::INSTALL,
-                    "Upgrade" => usage |= Usage::UPGRADE,
-                    _ => {}
-                }
-
-                if usage == Usage::NONE {
-                    usage = Usage::ALL
-                }
+        if !self.chroot {
+            for dep in &self.assume_installed {
+                alpm.add_assume_installed(&Depend::new(dep.as_str()))?;
             }
-
-            db.set_usage(usage)?;
         }
 
-        #[cfg(feature = "git")]
-        alpm.set_parallel_downloads(1);
-
-        alpm.set_ignorepkgs(self.ignore.iter())?;
-        alpm.set_ignoregroups(self.ignore_group.iter())?;
-
-        alpm.set_logfile(&*self.pacman.log_file)?;
-
-        #[cfg(not(feature = "git"))]
-        if let Some(arch) = self.pacman.architecture.get(0) {
-            alpm.set_arch(arch.as_str());
+        for pkg in &self.ignore {
+            alpm.add_ignorepkg(pkg.as_str())?;
         }
 
-        #[cfg(feature = "git")]
-        alpm.set_architectures(self.pacman.architecture.iter())?;
-
-        alpm.set_noupgrades(self.pacman.no_upgrade.iter())?;
-
-        alpm.set_use_syslog(self.pacman.use_syslog);
+        for group in &self.ignore_group {
+            alpm.add_ignoregroup(group.as_str())?;
+        }
 
         self.alpm = Alpm::new(alpm);
         Ok(())
@@ -740,7 +744,7 @@ impl Config {
         if key == "Include" {
             let value = match value {
                 Some(value) => value,
-                None => bail!("value can not be empty for value '{}'", key),
+                None => bail!(tr!("value can not be empty for key '{}'", key)),
             };
 
             let ini = std::fs::read_to_string(value)?;
@@ -756,20 +760,35 @@ impl Config {
 
         let section = match &self.section {
             Some(section) => section.as_str(),
-            None => bail!("key '{}' does not belong to a section", key),
+            None => bail!(tr!("key '{}' does not belong to a section", key)),
         };
 
         match section {
             "options" => self.parse_option(key, value),
             "bin" => self.parse_bin(key, value),
-            _ => bail!("unknown section '{}', section"),
+            "env" => self.parse_env(key, value),
+            _ => Ok(()),
         }
     }
 
+    fn parse_env(&mut self, key: &str, value: Option<&str>) -> Result<()> {
+        let value = value.context(tr!("key can not be empty"))?;
+
+        ensure!(!key.is_empty(), tr!("key can not be empty"));
+        ensure!(!key.contains('\0'), tr!("key can not contain null bytes"));
+        ensure!(
+            !value.contains('\0'),
+            tr!("value can not contain null bytes")
+        );
+
+        set_var(key, value);
+        Ok(())
+    }
+
     fn parse_bin(&mut self, key: &str, value: Option<&str>) -> Result<()> {
         let value = value
             .map(|s| s.to_string())
-            .ok_or_else(|| anyhow!("key can not be empty"))?;
+            .ok_or_else(|| anyhow!(tr!("key can not be empty")))?;
 
         let split = value.split_whitespace().map(|s| s.to_string());
 
@@ -789,7 +808,10 @@ impl Config {
             "SudoFlags" => self.sudo_flags.extend(split),
             "BatFlags" => self.bat_flags.extend(split),
             "FileManagerFlags" => self.fm_flags.extend(split),
-            _ => eprintln!("error: unknown option '{}' in section [bin]", key),
+            _ => eprintln!(
+                "{}",
+                tr!("error: unknown option '{}' in section [bin]", key)
+            ),
         };
 
         Ok(())
@@ -821,12 +843,12 @@ impl Config {
             "UseAsk" => self.use_ask = true,
             "NewsOnUpgrade" => self.news_on_upgrade = true,
             "DevelSuffixes" => {
-                let value = value.ok_or_else(|| anyhow!("key can not be empty"))?;
+                let value = value.ok_or_else(|| anyhow!(tr!("key can not be empty")))?;
                 let split = value.split_whitespace().map(|s| s.to_string());
                 self.devel_suffixes.extend(split);
             }
             "NoWarn" => {
-                let value = value.ok_or_else(|| anyhow!("key can not be empty"))?;
+                let value = value.ok_or_else(|| anyhow!(tr!("key can not be empty")))?;
                 let split = value.split_whitespace().map(|s| s.to_string());
                 for word in split {
                     self.no_warn_builder.add(Glob::new(&word)?);
@@ -847,7 +869,19 @@ impl Config {
                     self.chroot_dir = p.into();
                 }
             }
-            "MovePkgs" => self.move_pkgs = true,
+            "Sign" => {
+                self.sign = match value {
+                    Some(v) => Sign::Key(v.to_string()),
+                    None => Sign::Yes,
+                }
+            }
+            "SignDb" => {
+                self.sign_db = match value {
+                    Some(v) => Sign::Key(v.to_string()),
+                    None => Sign::Yes,
+                }
+            }
+
             _ => ok1 = false,
         }
 
@@ -858,7 +892,7 @@ impl Config {
         let has_value = value.is_some();
         let value = value
             .map(|s| s.to_string())
-            .ok_or_else(|| anyhow!("value can not be empty for value '{}'", key));
+            .ok_or_else(|| anyhow!(tr!("value can not be empty for key '{}'", key)));
 
         match key {
             "AurUrl" => self.aur_url = value?.parse()?,
@@ -874,9 +908,15 @@ impl Config {
         };
 
         if !(ok1 || ok2) {
-            eprintln!("error: unknown option '{}' in section [options]", key)
+            eprintln!(
+                "{}",
+                tr!("error: unknown option '{}' in section [options]", key)
+            )
         } else {
-            ensure!(ok1 || has_value, "option '{}' does not take a value", key);
+            ensure!(
+                ok1 || has_value,
+                tr!("option '{}' does not take a value", key)
+            );
         }
         Ok(())
     }
@@ -891,12 +931,7 @@ impl Config {
     }
 }
 
-fn help() {
-    let help = include_str!("../help");
-    print!("{}", help);
-}
-
-fn version() {
+pub fn version() {
     let ver = option_env!("PARU_VERSION").unwrap_or(env!("CARGO_PKG_VERSION"));
     print!("paru v{}", ver);
     #[cfg(feature = "git")]
@@ -915,19 +950,20 @@ fn reopen_stdin() -> Result<()> {
     Ok(())
 }
 
-fn question(question: &mut Question) {
-    let c = COLORS.get().unwrap();
+fn question(question: AnyQuestion, data: &mut (bool, Colors)) {
+    let no_confirm = data.0;
+    let c = data.1;
 
-    match question {
-        Question::SelectProvider(question) => {
+    match question.question() {
+        Question::SelectProvider(mut question) => {
             let providers = question.providers();
             let len = providers.len();
 
             println!();
-            let prompt = format!(
-                "There are {} providers available for {}:",
-                len,
-                question.depend()
+            let prompt = tr!(
+                "There are {n} providers available for {pkg}:",
+                n = len,
+                pkg = question.depend()
             );
             print!("{} {}", c.action.paint("::"), c.bold.paint(prompt));
 
@@ -939,7 +975,7 @@ fn question(question: &mut Question) {
                     println!(
                         "\n{} {} {}:",
                         c.action.paint("::"),
-                        c.bold.paint("Repository"),
+                        c.bold.paint(tr!("Repository")),
                         color_repo(c.enabled, pkg_db.name())
                     );
                     print!("    ");
@@ -947,23 +983,34 @@ fn question(question: &mut Question) {
                 print!("{}) {}  ", n + 1, pkg.name());
             }
 
-            let index = get_provider(len);
+            let index = get_provider(len, no_confirm);
             question.set_index(index as i32);
         }
-        Question::InstallIgnorepkg(question) => {
+        Question::InstallIgnorepkg(mut question) => {
             question.set_install(true);
         }
         _ => (),
     }
 }
 
-#[cfg(feature = "git")]
-fn download(filename: &str, event: DownloadEvent) {
-    match event {
+fn download(filename: &str, event: AnyDownloadEvent, _: &mut ()) {
+    match event.event() {
         DownloadEvent::Init(_) => println!("  syncing {}...", filename),
         DownloadEvent::Completed(c) if c.result == DownloadResult::Failed => {
-            println!("  failed to sync {}", filename);
+            printtr!("  failed to sync {}", filename);
         }
         _ => (),
     }
 }
+
+fn log(level: LogLevel, msg: &str, color: &mut Colors) {
+    let err = color.error;
+    let warn = color.warning;
+
+    match level {
+        LogLevel::WARNING => eprint!("{} {}", warn.paint("::"), msg),
+        LogLevel::ERROR => eprint!("{} {}", err.paint("error:"), msg),
+        LogLevel::DEBUG if alpm_debug_enabled() => eprint!("debug: <alpm> {}", msg),
+        _ => (),
+    }
+}
