@@ -1,10 +1,13 @@
 use crate::config::{Colors, Config, Mode, SortMode, YesNoAll};
 use crate::fmt::print_indent;
+use crate::printtr;
+use crate::RaurHandle;
 
 use std::collections::{HashMap, HashSet};
 use std::fs::read_dir;
 use std::io::Write;
 use std::iter::FromIterator;
+use std::path::Path;
 use std::process::{Command, Stdio};
 use std::result::Result as StdResult;
 
@@ -17,6 +20,7 @@ use indicatif::{ProgressBar, ProgressStyle};
 use kuchiki::traits::*;
 use raur::{ArcPackage as Package, Raur};
 use srcinfo::Srcinfo;
+use tr::tr;
 use url::Url;
 
 #[derive(Debug, Clone, Default)]
@@ -76,9 +80,9 @@ impl<'a> Warnings<'a> {
         if !self.missing.is_empty() {
             let b = color.bold;
             let e = color.error;
-            let len = ":: packages not in the AUR: ".len();
-            print!("{} {}", e.paint("::"), b.paint("Packages not in the AUR: "));
-            print_indent(Style::new(), len, 4, cols, "  ", &self.missing);
+            let msg = tr!("packages not in the AUR: ");
+            print!("{} {}", e.paint("::"), b.paint(&msg));
+            print_indent(Style::new(), msg.len() + 3, 4, cols, "  ", &self.missing);
         }
         self
     }
@@ -87,9 +91,9 @@ impl<'a> Warnings<'a> {
         if !self.ood.is_empty() {
             let b = color.bold;
             let e = color.error;
-            let len = ":: marked out of date: ".len();
-            print!("{} {}", e.paint("::"), b.paint("Marked out of date: "));
-            print_indent(Style::new(), len, 4, cols, "  ", &self.ood);
+            let msg = tr!("marked out of date: ");
+            print!("{} {}", e.paint("::"), b.paint(&msg));
+            print_indent(Style::new(), msg.len() + 3, 4, cols, "  ", &self.ood);
         }
         self
     }
@@ -98,9 +102,9 @@ impl<'a> Warnings<'a> {
         if !self.orphans.is_empty() {
             let b = color.bold;
             let e = color.error;
-            let len = ":: orphans: ".len();
-            print!("{} {}", e.paint("::"), b.paint("Orphans: "));
-            print_indent(Style::new(), len, 4, cols, "  ", &self.orphans);
+            let msg = tr!("orphans: ");
+            print!("{} {}", e.paint("::"), b.paint(&msg));
+            print_indent(Style::new(), msg.len() + 3, 4, cols, "  ", &self.orphans);
         }
         self
     }
@@ -113,7 +117,7 @@ impl<'a> Warnings<'a> {
 }
 
 pub async fn cache_info_with_warnings<'a, S: AsRef<str> + Send + Sync>(
-    raur: &raur::Handle,
+    raur: &RaurHandle,
     cache: &'a mut raur::Cache,
     pkgs: &'a [S],
     ignore: &[String],
@@ -169,7 +173,11 @@ pub async fn getpkgbuilds(config: &mut Config) -> Result<i32> {
         let aur = aur.iter().map(|t| t.pkg).collect::<Vec<_>>();
         let action = config.color.action;
         let bold = config.color.bold;
-        println!("{} {}", action.paint("::"), bold.paint("Querying AUR..."));
+        println!(
+            "{} {}",
+            action.paint("::"),
+            bold.paint(tr!("Querying AUR..."))
+        );
         let warnings =
             cache_info_with_warnings(&config.raur, &mut config.cache, &aur, &config.ignore).await?;
         if !warnings.missing.is_empty() {
@@ -192,12 +200,14 @@ pub async fn getpkgbuilds(config: &mut Config) -> Result<i32> {
 
 fn repo_pkgbuilds<'a>(config: &Config, pkgs: &[Targ<'a>]) -> Result<i32> {
     let db = config.alpm.syncdbs();
+    let c = config.color;
+    let mut r = 0;
 
-    let cd = std::env::current_dir().context("could not get current directory")?;
+    let cd = std::env::current_dir().context(tr!("could not get current directory"))?;
     let asp = &config.asp_bin;
 
     if Command::new(asp).output().is_err() {
-        bail!("can not get repo packages: asp is not installed");
+        bail!(tr!("can not get repo packages: asp is not installed"));
     }
 
     let cd = read_dir(cd)?
@@ -218,20 +228,52 @@ fn repo_pkgbuilds<'a>(config: &Config, pkgs: &[Targ<'a>]) -> Result<i32> {
     }
 
     if !missing.is_empty() {
-        let len = ":: Missing ABS packages ".len();
-        print!("{} Missing ABS packages ", config.color.error.paint("::"));
-        print_indent(config.color.base, len, 3, config.cols, "  ", &missing);
+        let msg = tr!("Missing ABS packages ");
+        print!("{} {} ", config.color.error.paint("::"), msg);
+        print_indent(
+            config.color.base,
+            msg.len() + 3,
+            3,
+            config.cols,
+            "  ",
+            &missing,
+        );
     }
 
     for (n, pkg) in ok.into_iter().enumerate() {
         print_download(config, n + 1, pkgs.len(), pkg);
-        let action = if cd.contains(pkg) { "update" } else { "export" };
 
         let ret = Command::new(asp)
-            .arg(action)
+            .arg("update")
             .arg(pkg)
             .output()
-            .with_context(|| format!("failed to run: {} {} {}", asp, action, pkg))?;
+            .with_context(|| format!("{} {} upadate {}", tr!("failed to run:"), asp, pkg))?;
+
+        ensure!(
+            ret.status.success(),
+            "{}",
+            String::from_utf8_lossy(&ret.stderr).trim()
+        );
+
+        if cd.contains(pkg) {
+            if !Path::new(pkg).join("PKGBUILD").exists() {
+                println!(
+                    "{} {} {}",
+                    c.warning.paint("::"),
+                    tr!("does not contain PKGBUILD: skipping"),
+                    pkg
+                );
+                r = 1;
+                continue;
+            }
+            std::fs::remove_dir_all(pkg)?;
+        }
+
+        let ret = Command::new(asp)
+            .arg("export")
+            .arg(pkg)
+            .output()
+            .with_context(|| format!("{} {} export {}", tr!("failed to run:"), asp, pkg))?;
 
         ensure!(
             ret.status.success(),
@@ -240,18 +282,20 @@ fn repo_pkgbuilds<'a>(config: &Config, pkgs: &[Targ<'a>]) -> Result<i32> {
         );
     }
 
-    Ok(!missing.is_empty() as i32)
+    if !missing.is_empty() {
+        r = 1;
+    }
+    Ok(r)
 }
 
 pub fn print_download(_config: &Config, n: usize, total: usize, pkg: &str) {
     let total = total.to_string();
     println!(
-        " ({:>padding$}/{}) downloading: {}",
-        n,
-        total,
-        //config.color.action.paint("::"),
-        pkg,
+        " ({n:>padding$}/{total}) {}",
+        tr!("downloading: {pkg}", pkg),
         padding = total.len(),
+        n = n,
+        total = total,
     );
 }
 
@@ -270,11 +314,11 @@ pub async fn aur_pkgbuilds(config: &Config, bases: &Bases) -> Result<()> {
     println!(
         "\n{} {}",
         action.paint("::"),
-        bold.paint("Downloading PKGBUILDs...")
+        bold.paint(tr!("Downloading PKGBUILDs..."))
     );
 
     if bases.bases.is_empty() {
-        println!(" PKGBUILDs up to date");
+        printtr!(" PKGBUILDs up to date");
         return Ok(());
     }
 
@@ -426,6 +470,7 @@ fn split_repo_aur_pkgbuilds<'a, T: AsTarg>(
 ) -> (Vec<Targ<'a>>, Vec<Targ<'a>>) {
     let mut local = Vec::new();
     let mut aur = Vec::new();
+    let db = config.alpm.syncdbs();
 
     for targ in targets {
         let targ = targ.as_targ();
@@ -442,19 +487,15 @@ fn split_repo_aur_pkgbuilds<'a, T: AsTarg>(
             } else {
                 aur.push(targ);
             }
-        } else if config
-            .alpm
-            .syncdbs()
-            .find_target_satisfier(targ.pkg)
-            .is_some()
-            || config
-                .alpm
-                .syncdbs()
-                .iter()
-                .filter(|db| targ.repo.is_none() || db.name() == targ.repo.unwrap())
-                .any(|db| db.group(targ.pkg).is_ok())
-        {
-            local.push(targ);
+        } else if let Ok(pkg) = db.pkg(targ.pkg) {
+            if matches!(
+                pkg.db().unwrap().name(),
+                "testing" | "community-testing" | "core" | "extra" | "community" | "multilib"
+            ) {
+                local.push(targ);
+            } else {
+                aur.push(targ);
+            }
         } else {
             aur.push(targ);
         }
@@ -474,7 +515,10 @@ pub async fn show_pkgbuilds(config: &mut Config) -> Result<i32> {
         let asp = &config.asp_bin;
 
         if Command::new(asp).output().is_err() {
-            eprintln!("{} is not installed: can not get repo packages", asp);
+            eprintln!(
+                "{}",
+                tr!("{} is not installed: can not get repo packages", asp)
+            );
             return Ok(1);
         }
 
@@ -483,7 +527,7 @@ pub async fn show_pkgbuilds(config: &mut Config) -> Result<i32> {
                 .arg("update")
                 .arg(&pkg.pkg)
                 .output()
-                .with_context(|| format!("failed to run: {} update {}", asp, pkg))?;
+                .with_context(|| format!("{} {} update {}", tr!("failed to run:"), asp, pkg))?;
 
             ensure!(
                 ret.status.success(),
@@ -496,7 +540,7 @@ pub async fn show_pkgbuilds(config: &mut Config) -> Result<i32> {
                     .arg("show")
                     .arg(&pkg.pkg)
                     .output()
-                    .with_context(|| format!("failed to run: {} show {}", asp, pkg))?;
+                    .with_context(|| format!("{} {} show {}", tr!("failed to run:"), asp, pkg))?;
 
                 ensure!(
                     output.status.success(),
@@ -510,9 +554,12 @@ pub async fn show_pkgbuilds(config: &mut Config) -> Result<i32> {
                     .arg("show")
                     .arg(&pkg.pkg)
                     .status()
-                    .with_context(|| format!("failed to run: {} show {}", asp, pkg))?;
+                    .with_context(|| format!("{} {} show {}", tr!("failed to run:"), asp, pkg))?;
 
-                ensure!(ret.success(), "asp returned {}", ret.code().unwrap_or(1));
+                ensure!(
+                    ret.success(),
+                    tr!("asp returned {}", ret.code().unwrap_or(1))
+                );
             }
             let _ = stdout.write_all(b"\n");
         }
